#define _GNU_SOURCE

#include <assert.h>
#include <errno.h>
#include <stdbool.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>
#include "config.h"
#include "logr.h"
#include "opts.h"

/* Initialize the atexit function to 'do nothing'.
 */
void (*atexit_fp) (void) = NULL;
/*---------------------------------------------------------------------------*/

/* Send a log message either to a stream or to syslog.
 */
void _vlogg
(
 const bool is_err,
 const unsigned char chattiness,
 const char *template,
 va_list ap
)
{
   /* Error (and warnings) are always at chattiness zero.
    */
   assert(!is_err || 0 == chattiness);

   if (opts.is_attached)
   {
      /* Probably being run for debug purposes so use the standard
       * streams.
       */
      FILE *stream = ( ((0 == chattiness) || is_err) ? stderr : stdout );
      fprintf(stream, "%s: ", PACKAGE_NAME);
      vfprintf(stream, template, ap);
   }
   else
   {
      /* Running as a daemon so use syslog.
       */
      openlog(PACKAGE_NAME, LOG_PID, LOG_USER);
      int priority =
         (
          is_err ? LOG_ERR :
          (0 == chattiness ? LOG_WARNING : LOG_INFO)
         );
      vsyslog(priority, template, ap);
      closelog();
   }
   return;
}
/*---------------------------------------------------------------------------*/

void
logg (const unsigned char chattiness, const char *template, ...)
{
   if (chattiness <= opts.verbosity)
   {
      va_list ap;
      va_start(ap, template);
      _vlogg(false, chattiness, template, ap);
      va_end(ap);
   }
   return;
}
/*---------------------------------------------------------------------------*/

void
logg_exit (const char *template, ...)
{
   va_list ap;
   va_start(ap, template);
   _vlogg(true, 0, template, ap);
   va_end(ap);
   if (NULL != atexit_fp)
   {
      logg(3, "%s calling atexit()...\n", __func__);
      if (0 != atexit(atexit_fp))
      {
         logg(0, "atexit failed on %s\n", strerror(errno));
      }
      logg(3, "%s done calling atexit().\n", __func__);
   };
   logg(3, "%s about to call exit().\n", __func__);
   exit(EXIT_FAILURE);
}
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
