#ifndef LOGR_H
#define LOGR_H

/* For clients to set an atexit function, triggered by calling
   logg_exit(), see below.
 */
extern void (*atexit_fp) (void);

/* Write a message.  If chattiness is zero, write it as a warning,
 * else as information.
 *
 * Following the chattiness parameter, the signature imitates printf()
 * and friends.
 */
void
logg (const unsigned char chattiness, const char *template, ...)
   __attribute__ ((format (printf, 2, 3)));

/* Write a message as an error and exit with failure.  Before calling
   exit() it check whether atexit_fp (above) is set, and if so calls
   atexit(atexit_fp).  This way the client can choose some atexit
   function and later clear it (by setting atexit_fp to NULL), which
   is not possible with atexit() itself.
 */
void
logg_exit (const char *template, ...)
   __attribute__ ((format (printf, 1, 2)));

#endif
