#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <signal.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "daem.h"
#include "lock.h"
#include "logr.h"
#include "opts.h"
#include "pger.h"

int
main (int argc, char **argv)
{
   /* Fill out the opts structure from the command line.
    */
   init_opts_from_cmdline(argc, argv);
   logg(2,
        "is_attached `%d';\n"
        "connect `%s';\n"
        "pidfile_name `%s';\n"
        "run `%s';\n"
        "sql `%s';\n"
        "timeout `%d';\n"
        "verbosity `%d';\n"
        "notification name `%s'.\n",
        opts.is_attached,
        opts.connect,
        opts.pidfile_name,
        opts.run,
        opts.sql,
        opts.timeout_sec,
        opts.verbosity,
        opts.ntn_name);

   /* Are we willing and able to daemonize?
    */
   bool is_daemonaic = (!opts.is_attached && 1 < getppid());
   if (is_daemonaic) daemonize();

   /* At this point, is_daemonaic being true means we're the child
      since the parent does not return from daemonize().  Either way,
      continue with setting up.
    */
   lock_pidfile(opts.pidfile_name);
   connect_to_db();

   /* Setup finished.  Signal the parent (if daemonized) that we setup
      ok.
   */
   if (is_daemonaic) send_signal_child_ok();

   /* Listen and respond 'forever'.
    */
   listen_loop();
   logg(1, "leaving %s.\n", __func__);

   return EXIT_SUCCESS;
}
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
